/* samples/execcc_sample.p  Prompt for (and bill) CC    SAMPLE PROGRAM
            DY Tri-8, 04/22/2002
*/

/**************************************************************************
$Id: execcc_sample.p,v 1.4 2004/07/02 15:31:15 dan Exp $

  Author: Dan Yost <dan@tri8.com>
    Date: 04/22/2002

   USAGE: execcc ( input-output temp-table trExec,
		   output log_trans-exec ) 

  OUTPUT: temp-table trExec, which has been updated
	  trans-exec, which indicates whether the transaction (authorization)
	  was executed (false if f4 was pressed, for example)

  DESCRIPTION:
    This SAMPLE PROGRAM prompts the user for CC details and runs a credit card 
    authorization.  The action (CCA_PURCHASE) and other data are hard-coded
    merely for the sake of example.  You may create the trExec record before
    calling this program, set certain non-user-editable fields, and then pass 
    it to this program, which will display relevant portions and receive user
    input.  This merely shows how one might retrieve user input and set up a
    credit card transaction.

**************************************************************************/

 /************************************************************************
 * COPYRIGHT Tri-8, Inc. This work is unpublished.  ALL RIGHTS RESERVED. * 
 ************************************************************************/

/*************************** DEFINITIONS *********************************/
{tri8osc/oscapi}

def input-output parameter table for trExec.
def output parameter trans-exec as log no-undo.

def var res-text as char no-undo.
def var do-proc as log no-undo.
def var card-auth as log no-undo.
def var re-prompt as log no-undo.
def var cust-name as char format 'x(30)' no-undo.
def var force-negative as log no-undo.
def var frame-title as char no-undo.
def var vauth-obtained as log no-undo.

form
  skip(1)
  trExec.cardNo colon 10 skip
  trExec.expDate format "99/99" colon 10 skip
  trExec.amount colon 10 skip
  trExec.CHName colon 10 skip
  trExec.CHAddress colon 10 skip
  trExec.CHZip colon 10 skip
  trExec.cvv2 colon 10 skip
  trExec.apprCode colon 10 skip
  trExec.comment colon 10 view-as editor size 35 by 3
  skip(1)
  with overlay centered row 6 frame trExec-fr side-labels
       title frame-title.

form
  " Visa/MasterCard: 1-800-991-9911  Merchant #9999-999-99" skip
  "American Express: 1-800-528-2121  Merchant #1234-5678" 
  with overlay row 1 centered frame voice-auth-fr 
       title "Voice Authorizations".

/*************************** MAIN BLOCK **********************************/

CCBLK: 
do transaction
on endkey undo CCBLK, retry CCBLK:

    if retry
    then do:
	/* quit on F4, but remain if re-prompt (user answered no to "sure?") */
	if re-prompt ne true
	then do:
	    hide frame trExec-fr no-pause.
	    hide frame voice-auth-fr no-pause.
	    return.
	end.
    end.

    trans-exec = false.  
    re-prompt = false.

    find trExec exclusive-lock no-error.

    if not available trExec 
    then create trExec.

    assign trExec.enteredBy = "Joe"
	   trExec.action    = CCA_PURCHASE
	   trExec.timeout   = OSCConfig.CCTimeout

	   /* this was formerly an input parameter, but we assign it now */
	   force-negative     = if trExec.action = CCA_CREDIT
	                        then true
				else false
           
	   frame-title        = if trExec.action = CCA_VOICEAUTH
	                        then " Credit Card Voice Authorization "
				else " Credit Card Information ".

    if trExec.action = CCA_VOICEAUTH
    then view frame voice-auth-fr.

    display trExec.cardNo
	    trExec.expDate
	    trExec.amount
	    trExec.CHName
	    trExec.CHAddress
	    trExec.CHZip
	    trExec.cvv2
	    trExec.apprCode 
	    trExec.comment
	    with frame trExec-fr.

    if trExec.action = CCA_VOICEAUTH
    then do:
        vauth-obtained = true.

	message "Please dial the appropriate phone number.~n~n"
	        "Was the card approved?"
		view-as alert-box information 
		        buttons yes-no update vauth-obtained.

        if vauth-obtained = true
	then update trExec.apprCode 
	            trExec.comment
	            with frame trExec-fr.
        else do:
	    assign frame-title = " Credit Card Voice Authorization DECLINED "
	           trExec.apprCode   = "DECLINED"
		   no-error .

	    hide frame trExec-fr no-pause.
	    view frame trExec-fr.

            display trExec.apprCode
	            with frame trExec-fr.
            
	    update trExec.comment 
	           with frame trExec-fr.
	end. /* not vauth-obtained */

    end. /* if action = CCA_VOICEAUTH */
    else do:
	update trExec.cardNo
	       trExec.expDate
	       trExec.amount
	       trExec.CHName
	       trExec.CHAddress
	       trExec.CHZip
	       trExec.cvv2
	       trExec.comment
	       with frame trExec-fr.
    end. /* else action <> CCA_VOICEAUTH */

    /* repeat until valid data (you may define your own validation functions) */
    repeat while not validCC( input table trExec,
                               output res-text)
                 or (force-negative = true and trExec.amount > 0)
        on endkey undo CCBLK, return:


        /* note that we force negative amounts merely to make sure that the
	// user understands that this is a REFUND/CREDIT */
	if force-negative
	   and trExec.amount > 0
	then res-text = msg-append(res-text,
			           "amount must be negative (for a refund)").
        
	message replace(res-text,MSGDELIM,"~n") view-as alert-box error.

	if trExec.action = CCA_VOICEAUTH
	then do:
	    update trExec.apprCode
		   trExec.comment
		   with frame trExec-fr.
	end. /* if action = CCA_VOICEAUTH */
	else do:
	    update trExec.cardNo
		   trExec.expDate
		   trExec.amount
		   trExec.CHName
		   trExec.CHAddress
		   trExec.CHZip
		   trExec.cvv2
		   trExec.comment
		   with frame trExec-fr.
	end. /* else action <> CCA_VOICEAUTH */

    end. /* repeat while not validCC */

    hide frame trExec-fr no-pause.
    hide frame voice-auth-fr no-pause.
    do-proc = false. 
    
    message "Are you sure?" 
       view-as alert-box question buttons yes-no 
       title "Process credit card" update do-proc.
    
    if not do-proc = true 
    then do:
	re-prompt = true.
	undo CCBLK, retry CCBLK.
    end.
    else do:
        display skip(1) "   Processing..." skip(1)
           with frame prc overlay centered row 13 width 20 title "Please wait" .

        if trExec.action = CCA_VOICEAUTH
	   and vauth-obtained <> true
        then do:
	    assign trExec.authStatus = CCS_DECL
	           trans-exec = true.

	    hide frame prc no-pause.

	    message "voice declination registered".

	    return.
	end. /* if voice auth was declined or not correctly answered */
                   
        /* perform credit card authorization */                        


	/* run something like cctrans.sample if you have developed an interface
	// that will include permanent DB storage.  as-is, we'll call 
	// tri8osc/oscengin directly. */

	/**** sample
        run tri8osc/cctrans_sample ( input-output table trExec,
			             input no ) no-error.
        ******/
	run tri8osc/oscengin ( input-output table trExec ).

        find trExec exclusive-lock.

	/* indicate that we HAVE executed a cctrans call */
        trans-exec = true.

        hide frame prc no-pause.
                              
        case trExec.authStatus:
            when CCS_AUTHONLY or when CCS_RTS then do:
	        case trExec.action:
		when CCA_PURCHASE
		then message "card APPROVED, auth code"  
		             trim(trExec.apprCode).

		when CCA_CREDIT
		then message "credit registered".

		when CCA_VOICEAUTH
		then message "voice authorization registered".
		end case. /* trExec.action */
            end. /* when CCS_AUTHONLY or when CCS_RTS */
                    
            when CCS_DECL then do:
                message "card DECLINED, reason given:"  
		        trim(trExec.authDetails) .
                return.
            end. /* when "denied" */

	    when CCS_AVSR then do:
		message "AVS rejected: "
		        trim(trExec.authDetails) .
                return.
	    end. /* when "AVS reject" */
                    
            when CCS_CALL then do:
		message "Card requires voice authorization:"
			trim(trExec.authDetails).

                return.
            end. /* when CCS_CALL */

	    when CCS_SUBERR
	    then do:
		message "An error has occurred"  trim(trExec.authDetails)
		        "~nThis card HAS been submitted for billing--DO NOT"
		        "re-submit the card".

		return. 
	    end. /* when CCS_SUBERR */

	    when CCS_SYSERR
	    then do:
                message "A system error has occurred:" 
			trim(trExec.authDetails).

                return.
	    end. /* when CCS_SYSERR */

	    when CCS_ERR
	    then do:
		message "Error:" trim(trExec.authDetails).

		return.
	    end. /* when CCS_ERR */

            otherwise do:
                message "Uncaught authStatus: " 
		   + CCChar(trExec.authStatus) + "!"
                   view-as alert-box information title "ERROR" .
            end. /* otherwise */
        end case. /* auth-res */
                
    end. /* else do-proc */
end. /* CCBLK */
              
